unit Render;

interface

uses
  Classes, Graphics, Fields, RasterField, Globals;

type TFieldRenderer=class
                    private
                      Cache: TBitmap;  // Cache-Bitmap
                      FZoom: integer;  // Zoomstufe (= Feldgre)
                      procedure SetZoom(Value:integer);
                    public
                      // Rendering-Einstellungen
                      FieldSize           : integer;  // Gre eines Feldes
                      PlayerLineWidth     : integer;  // Breite einer Linie
                      EmptyFieldColor     : TColor;   // leeres Feld, Hintergr.
                      EmptyFieldLines     : TColor;   // leeres Feld, Linien
                      UsedFieldColor      : TColor;   // Nutzfeld, Hintergrund
                      UsedFieldLines      : TColor;   // Nutzfeld, Linien
                      CircleRadius        : integer;  // Anfasserkreis-Radius
                      CircleLineColor     : TColor;   // Anfasserkreis-Randfarbe
                      PlayerColors: array[OWNER_EMPTY..OWNER_DEAD] of TColor;
                        // Farben der Spieler
                      HighlightCircleColor: TColor;   // aktiver Anfasser (Fll)
                      HighlightCircleLines: TColor;   // aktiver Anfasser (Rand)
                      // Rendering-Flags
                      ShowField           : boolean;
                        // Feldlinien und -Anfasser anzeigen?
                      ShowPossibleOwner   : boolean;
                        // PossibleOwner anzeigen, wenn mglich?
                      ConnectLines        : boolean;
                        // Linien als Verbindung der umliegenden Punkte anzeigen
                        // (true, Standard) oder nur "Stummel" um den Mittel-
                        // punkt? (false)
                      // Konstruktor/Destruktor
                      constructor Create;
                      destructor Destroy; override;
                      // Form & Farbe
                      property Zoom:integer read FZoom write SetZoom;
                      // sinnvolle Funktionen :)
                      procedure RenderField(  // Feld mittels Cache rendern
                        Field: TField; Raster: TRaster; Canvas: TCanvas);
                      procedure RenderFieldDirect(  // Feld direkt rendern
                        Field: TField; Raster: TRaster; Canvas: TCanvas);
                      procedure HighlightNode(X, Y:integer; Canvas: TCanvas);
                        // einen Knoten hervorheben
                    end;

implementation

constructor TFieldRenderer.Create;
begin
  inherited Create;
  Cache := nil;
  // Voreinstellungen
  SetZoom(24);
  EmptyFieldColor := $70B0C0;
  EmptyFieldLines := $A0CCD4;
  UsedFieldColor := $888078;
  UsedFieldLines := $404040;
  CircleLineColor := $000000;
  PlayerColors[OWNER_EMPTY] := $C0C0C0;
  PlayerColors[OWNER_EMPTY+1] := $0000FF;
  PlayerColors[OWNER_EMPTY+2] := $00FF00;
  PlayerColors[OWNER_DEAD] := 0;
  HighlightCircleColor := $FFC080;
  HighlightCircleLines := $683828;
  ShowField := true;
  ShowPossibleOwner := false;
  ConnectLines := true;
end;

destructor TFieldRenderer.Destroy;
begin
  inherited Destroy;
end;

procedure TFieldRenderer.SetZoom(Value:integer);
begin
  if Value<8 then Value:=8;
  FZoom := Value;
  FieldSize := Value;
  PlayerLineWidth := Value DIV 8;
  CircleRadius := (Value DIV 5) OR 1;
end;

procedure TFieldRenderer.RenderField
          (Field: TField; Raster: TRaster; Canvas: TCanvas);
var NewWidth, NewHeight: integer;
begin
  // Spielfeldgre bestimmen
  NewWidth  := FieldSize * Raster.Width;
  NewHeight := FieldSize * Raster.Height;
  // Gre gendert oder noch gar kein Cache da ==> Cache-Bitmap neu anlegen
  if not(Assigned(Cache)) OR
     (Cache.Width <> NewWidth) OR (Cache.Height <> NewHeight) then begin
    if Assigned(Cache) then Cache.Free;
    Cache := TBitmap.Create;
    with Cache do begin
      PixelFormat := pfDevice;
      Width  := NewWidth;
      Height := NewHeight;
    end;
  end;
  // Feld rendern
  RenderFieldDirect(Field, Raster, Cache.Canvas);
  Canvas.Draw(0, 0, Cache);
end;

procedure TFieldRenderer.RenderFieldDirect
          (Field: TField; Raster: TRaster; Canvas: TCanvas);
var MX, MY: integer;  // maximale Koordinatenwerte
    BX, BY: integer;  // Basiskoordinten des Gitterpunktes
    X,  Y : integer;  // Zhler
    i: integer;       // temporre Variable
    LLL: integer;      // "logical line length"
begin with Canvas do begin
  MX := Raster.Width - 1;
  MY := Raster.Height - 1;
  // Feldhintergrund
  Brush.Style := bsSolid;
  for Y := 0 to MY do
    for X := 0 to MX do begin
      if Raster[X, Y] then Brush.Color := UsedFieldColor
                      else Brush.Color := EmptyFieldColor;
      FillRect(Rect( 1 + X  * FieldSize,  1 + Y  * FieldSize,
                    (1 + X) * FieldSize, (1 + Y) * FieldSize));
    end;
  // Auenrahmen
  with Pen do begin
    Style := psSolid;
    Width := 1;
    Color := EmptyFieldLines;
  end;
  MoveTo(0, 0);
  LineTo((MX + 1) * FieldSize, 0);
  LineTo((MX + 1) * FieldSize, (MY + 1) * FieldSize);
  LineTo(0, (MY + 1) * FieldSize);
  LineTo(0,0);
  // Innenlinien
  for Y := 0 to MY do
    for X := 0 to (MX - 1) do begin
      if (Raster[X, Y] OR Raster[X + 1, Y])
        then Pen.Color :=  UsedFieldLines
        else Pen.Color := EmptyFieldLines;
      MoveTo((X + 1) * FieldSize, Y * FieldSize);
      LineTo((X + 1) * FieldSize, (Y + 1) * FieldSize);
    end;
  for Y := 0 to (MY - 1) do
    for X := 0 to MX do begin
      if (Raster[X, Y] OR Raster[X, Y + 1])
        then Pen.Color :=  UsedFieldLines
        else Pen.Color := EmptyFieldLines;
      MoveTo(X * FieldSize, (Y + 1) * FieldSize);
      LineTo((X + 1) * FieldSize, (Y + 1) * FieldSize);
    end;
  if not ShowField then exit;
  // Spielerlinien
  if ConnectLines then LLL := FieldSize
                  else LLL := FieldSize DIV 2;
  with Pen do begin
    Style := psSolid;
    Width := PlayerLineWidth;
  end;
  for Y := 0 to (MY-1) do begin
    BY := (Y + 1) * FieldSize;
    for X := 0 to (MX-1) do
      with Field.FieldPtr[X, Y]^ do begin
        BX := (X + 1) * FieldSize;
        i := Owner;
        if (i >= OWNER_EMPTY) AND (i < OWNER_DEAD) then begin
          Pen.Color := PlayerColors[i];
          i := Dirs;
          if (i AND DIR_H) <> 0 then begin
            MoveTo(BX - LLL, BY);
            LineTo(BX + LLL, BY);
          end;
          if (i AND DIR_V) <> 0 then begin
            MoveTo(BX, BY - LLL);
            LineTo(BX, BY + LLL);
          end;
          if (i AND DIR_D) <> 0 then begin
            MoveTo(BX - LLL, BY + LLL);
            LineTo(BX + LLL, BY - LLL);
          end;
          if (i AND DIR_A) <> 0 then begin
            MoveTo(BX - LLL, BY - LLL);
            LineTo(BX + LLL, BY + LLL);
          end;
        end;
      end;
  end;
  // Anfasserkreise
  with Pen do begin
    Style := psSolid;
    Width := 1;
    Color := CircleLineColor;
  end;
  Brush.Style := bsSolid;
  for Y := 0 to (MY-1) do begin
    BY := (Y + 1) * FieldSize;
    for X := 0 to (MX-1) do begin
      BX := (X + 1) * FieldSize;
      i := Field.FieldPtr[X, Y]^.Owner;
      if ShowPossibleOwner AND (i = OWNER_EMPTY) then
        i := Field.FieldPtr[X, Y]^.PossibleOwners;
      if (i >= OWNER_EMPTY) AND (i <= OWNER_DEAD) then begin
        Brush.Color := PlayerColors[i];
        Ellipse(BX - CircleRadius, BY - CircleRadius,
                BX + CircleRadius, BY + CircleRadius);
      end;
    end;
  end;
end; end;

procedure TFieldRenderer.HighlightNode(X, Y:integer; Canvas: TCanvas);
var BX, BY: integer;
begin
  with Canvas do begin
    BX := (X + 1) * FieldSize;
    BY := (Y + 1) * FieldSize;
    with Brush do begin
      Style := bsSolid;
      Color := HighlightCircleColor;
    end;
    with Pen do begin
      Style := psSolid;
      Width := 1;
      Color := HighlightCircleLines;
    end;
    Ellipse(BX - CircleRadius, BY - CircleRadius,
            BX + CircleRadius, BY + CircleRadius);
  end;
end;

end.
